# get command-line args
args <- commandArgs(TRUE)
file1 <- args[1]
file2 <- args[2]
# read benchmarks
bm1 <- read.table(file1, sep="\t", header=T)
bm2 <- read.table(file2, sep="\t", header=T)


# getting the differences for generic benchmarks
# new[1] contains the benchmark names, 2:5 the iters/time/mem/allocs
diff <- data.frame(new[1], new[2:5] - old[2:5])
# negative is better
better <- diff[labeleddiffs$Time < 0,]



# graphing both data frames (on top of each other)


# from https://cran.r-project.org/doc/contrib/Lemon-kickstart/kr_addat.html
# get the range (min, max y value) of the tables so we can plot them
# on the same scale
yrange <- range(c(bm1[, "Time"], bm2[, "Time"]))
# plot one graph
plot(Time ~ Turns, data=bm1, ylim=yrange)
# don't erase the old graph before drawing the new one
# (par() means "modify graphics params")
par(new=T)  # don't ask me why TRUE is DON'T make a new plot and FALSE is DO make a new plot
# plot the second graph on top of the first one with a different color (col=2)
# and don't overwrite the axes (axes=F)
plot(Time ~ Turns, data=bm2, ylim=yrange, col=2, axes=F)
# return setting to default
par(new=F)

# get the average of time
turnsf1 <- factor(bm1[,"Turns"])
meantimes1 <- tapply(bm1[,"Time"], turnsf1, mean)
turnsf2 <- factor(bm2[,"Turns"])
meantimes2 <- tapply(bm2[,"Time"], turnsf2, mean)
# and standard deviation
sdtimes1 <- tapply(bm1[,"Time"], turnsf1, sd)
sdtimes2 <- tapply(bm2[,"Time"], turnsf2, sd)

up1 <- meantimes1 + sdtimes1
down1 <- meantimes1 - sdtimes1
up2 <- meantimes2 + sdtimes2
down2 <- meantimes2 - sdtimes2

yrange <- range(c(up1, down1, up2, down2))
# plot means
# the "levels" of a factor are the unique values
# so [5, 10, 15, 20, ...]
levelsf1 <- levels(turnsf1)
levelsf2 <- levels(turnsf2)
plot(levelsf1, meantimes1, type="b", ylim=yrange, xlab="Turns", ylab="Time", col=1)
par(new=T)
plot(levelsf2, meantimes2, type="b", axes=F, ylim=yrange, xlab="Turns", ylab="Time", col=2)
# ^ type="b": plot points and lines
#arrows(levelsf1, down1, levelsf1, up1, angle=90, length=0.03, code=3)

par(new=T)
plot(levelsf1, up1, type="l", axes=F, ylim=yrange, xlab="Turns", ylab="Time", col=1)
par(new=T)
plot(levelsf2, up2, type="l", axes=F, ylim=yrange, xlab="Turns", ylab="Time", col=2)
# ^ type="l": plot line (no points)
par(new=T)
plot(levelsf1, down1, type="l", axes=F, ylim=yrange, xlab="Turns", ylab="Time", col=1)
par(new=T)
plot(levelsf2, down2, type="l", axes=F, ylim=yrange, xlab="Turns", ylab="Time", col=2)
par(new=F)

# tourneydiff
# assuming 1,2 and 5,6 are the parameter columns for black and white
# (i.e. 2 parameters for each)
# tourneydiff<-data.frame(tourney[1:2],
#                  tourney$BPieces-tourney$WPieces,
#                  tourney$BTimeTaken-tourney$WTimeTaken,
#                  tourney[5:6])
# names(tourneydiff)[3:4] <- c("DPieces", "DTimeTaken")

# getting wins vs total on a parameter
dfw <- function (df) { df[df$BPieces > df$WPieces & df$BTimeLeft > 0,] }
dfl <- function (df) { df[df$BPieces <= df$WPieces | df$BTimeLeft <= 0,] }
winr<-function (para, df, dfw) { t<-nrow(df[grep(para, df$BName),]); w<-nrow(dfw[grep(para, dfw$BName),]); list(c(w, t, w/t)) }
# use like
# winr("nlt=0", tb2m, tb2mw)
